import { StatsD } from 'hot-shots';
import * as os from 'os';

class Metrics {
  static getMetricClient() {
    return this.metricClient;
  }

  static setDefaultClient(client) {
    this.metricClient = client;
  }

}
Metrics.metricClient = void 0;

class Constants {}
Constants.MaxAttributes = 30;
Constants.MaxAttributeNameLength = 100;
Constants.MaxAttributeValueLength = 1024;
Constants.HistogramBucketAttributeName = 'histogram_bucket_bounds';
Constants.DefaultHistogramBuckets = [0, 5, 10, 25, 50, 75, 100, 250, 500, 1000];
Constants.DefaultTimerBuckets = [0, 10, 25, 50, 75, 100, 250, 500, 1000, 5000, 10000, 30000, 60000, 240000];

class Configuration {
  static initializeConfiguration(configJSON) {
    var _config$application, _config$domain, _config$module, _config$metrics, _config$metrics$hostn, _config$metrics$port, _config$metrics$enabl;

    let config;

    try {
      config = JSON.parse(configJSON != null ? configJSON : '{}');
    } catch (error) {
      return {
        application: this.defaultApp,
        domain: this.defaultDomain,
        module: this.defaultModule,
        metrics: {
          client: this.defaultMetricsClient,
          hostname: this.defaultMetricsHost,
          port: this.defaultMetricsPort,
          enabled: this.defaultMetricsEnabled
        }
      };
    }

    config.application = (_config$application = config.application) != null ? _config$application : this.defaultApp;
    config.domain = (_config$domain = config.domain) != null ? _config$domain : this.defaultDomain;
    config.module = (_config$module = config.module) != null ? _config$module : this.defaultModule;
    config.metrics = (_config$metrics = config.metrics) != null ? _config$metrics : {};
    config.metrics.client = config.metrics.client !== undefined ? config.metrics.client.toLowerCase() : this.defaultMetricsClient;
    config.metrics.hostname = (_config$metrics$hostn = config.metrics.hostname) != null ? _config$metrics$hostn : this.defaultMetricsHost;
    config.metrics.port = (_config$metrics$port = config.metrics.port) != null ? _config$metrics$port : this.defaultMetricsPort;
    config.metrics.enabled = (_config$metrics$enabl = config.metrics.enabled) != null ? _config$metrics$enabl : this.defaultMetricsEnabled;
    return config;
  }

}
Configuration.defaultApp = '';
Configuration.defaultDomain = '';
Configuration.defaultModule = '';
Configuration.defaultMetricsClient = 'statsd';
Configuration.defaultMetricsHost = '127.0.0.1';
Configuration.defaultMetricsPort = 8125;
Configuration.defaultMetricsEnabled = true;

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends.apply(this, arguments);
}

class Warnings {}
Warnings.MetricTypeMismatch = 'MetricTypeMismatch';
Warnings.TooManyAttributes = 'TooManyAttributes';
Warnings.AttributeKeyTooLong = 'AttributeKeyTooLong';
Warnings.AttributeValueTooLong = 'AttributeValueTooLong';
Warnings.MetricSkipped = 'MetricSkipped';
Warnings.NoDefaultNamespaceSet = 'NoDefaultNamespaceSet';
Warnings.InvalidAttributeKey = 'InvalidAttributeKey';
Warnings.InvalidAttributeValue = 'InvalidAttributeValue';
Warnings.FailedToSetHostname = 'FailedToSetHostname';

class DefaultLogger {
  warn(message) {
    process.stdout.write(message);
  }

}

class Logger {
  static getLogger() {
    if (this.logger === undefined) {
      this.logger = new DefaultLogger();
    }

    return this.logger;
  }

  static setDefaultLogger(logger) {
    this.logger = logger;
  }

}
Logger.logger = void 0;

class Metric {
  constructor(namespace, name, description, unit, attributes = {}) {
    this.namespace = void 0;
    this.name = void 0;
    this.description = void 0;
    this.unit = void 0;
    this.attributes = void 0;
    this.namespace = namespace;
    this.name = name;
    this.description = description != null ? description : '';
    this.unit = unit != null ? unit : '';
    this.attributes = attributes;
  }

  getNamespace() {
    return this.namespace;
  }

  getName() {
    return this.name;
  }

  getDescription() {
    return this.description;
  }

  getUnit() {
    return this.unit;
  }

  getAttributes() {
    return this.attributes;
  }

  getAttribute(attributeName) {
    return this.attributes[attributeName];
  }

  setAttribute(attributeName, attributeValue) {
    this.attributes[attributeName] = attributeValue;
  }

  removeAttribute(attributeName) {
    delete this.attributes[attributeName];
  }

  combineAttributes(attributes) {
    var _this$namespace$getMe, _this$namespace$getAt, _this$getAttributes;

    const combinedAttributes = _extends({}, (_this$namespace$getMe = this.namespace.getMetricClient().getAttributes()) != null ? _this$namespace$getMe : {}, (_this$namespace$getAt = this.namespace.getAttributes()) != null ? _this$namespace$getAt : {}, (_this$getAttributes = this.getAttributes()) != null ? _this$getAttributes : {}, attributes != null ? attributes : {});

    const cleanAttributes = {};
    let attributeCount = 0;

    for (const index in Object.keys(combinedAttributes)) {
      if (Object.prototype.hasOwnProperty.call(Object.keys(combinedAttributes), index)) {
        let attributeName = Object.keys(combinedAttributes)[index];
        let attributeValue = Object.values(combinedAttributes)[index];

        if (attributeName === null || attributeName === undefined || attributeName === '') {
          Logger.getLogger().warn(`${Warnings.InvalidAttributeKey}`);
          continue;
        }

        if (attributeValue === null || attributeValue === undefined || attributeValue === '') {
          Logger.getLogger().warn(`${Warnings.InvalidAttributeValue} - ${this.name}: ${attributeName}`);
          continue;
        }

        if (attributeName.length > Constants.MaxAttributeNameLength) {
          Logger.getLogger().warn(`${Warnings.AttributeKeyTooLong} - ${this.name}: ${attributeName}`);
          attributeName = attributeName.substring(0, Constants.MaxAttributeNameLength - 1);
        }

        if (attributeValue.length > Constants.MaxAttributeValueLength) {
          Logger.getLogger().warn(`${Warnings.AttributeValueTooLong} - ${this.name}: ${attributeValue}`);
          attributeValue = attributeValue.substring(0, Constants.MaxAttributeValueLength - 1);
        }

        if (attributeCount < Constants.MaxAttributes) {
          cleanAttributes[attributeName] = attributeValue.toString();
        }

        attributeCount++;
      }
    }

    if (attributeCount >= Constants.MaxAttributes) {
      Logger.getLogger().warn(`${Warnings.TooManyAttributes} - ${this.name}: ${attributeCount}`);
    }

    return cleanAttributes;
  }

  getFullName() {
    return `${this.namespace.getNamespacePrefix()}${this.name}`;
  }

}

class Counter extends Metric {
  constructor(namespace, name, description, unit, attributes = {}) {
    super(namespace, name, description, unit, attributes);
  }

  increment(value, attributes) {
    this.getNamespace().getMetricClient().incrementCounter(this.getFullName(), value, this.combineAttributes(attributes));
  }

}

class Gauge extends Metric {
  constructor(namespace, name, description, unit, attributes = {}) {
    super(namespace, name, description, unit, attributes);
  }

  record(value, attributes) {
    this.getNamespace().getMetricClient().recordGauge(this.getFullName(), value, this.combineAttributes(attributes));
  }

}

class Histogram extends Metric {
  constructor(namespace, name, description, unit, buckets = [], attributes = {}) {
    super(namespace, name, description, unit, attributes);
    this.defaultBuckets = Constants.DefaultHistogramBuckets;
    this.buckets = [];
    this.buckets = buckets;
  }

  record(value, buckets = [], attributes = {}) {
    const resolvedBuckets = this.resolveBuckets(buckets);
    const combinedAttributes = this.combineAttributes(attributes);
    this.getNamespace().getMetricClient().recordTimer(this.getFullName(), value, resolvedBuckets, combinedAttributes);
  }

  resolveBuckets(buckets) {
    if (buckets && buckets.length) {
      return buckets;
    }

    if (this.buckets && this.buckets.length) {
      return this.buckets;
    }

    return this.defaultBuckets;
  }

}

class Timer extends Histogram {
  constructor(namespace, name, description, unit, buckets = [], attributes = {}) {
    super(namespace, name, description, unit, buckets, attributes);
    this.defaultBuckets = Constants.DefaultTimerBuckets;
  }

  time(buckets = Constants.DefaultTimerBuckets, attributes = {}, fn, ...args1) {
    const startTime = new Date();
    let result;

    const logTimeSpent = () => {
      const combinedAttributes = _extends({}, attributes);

      combinedAttributes.success = success;
      const endTime = new Date();
      const duration = endTime.getTime() - startTime.getTime();
      this.record(duration, buckets, combinedAttributes);
    };

    let success = 'true';
    let synchronous = true;

    try {
      result = fn(...args1);

      if (this.isThenable(result)) {
        synchronous = false;
        void result.catch(() => {
          success = 'false';
        }).then(() => logTimeSpent());
      }
    } catch (error) {
      success = 'false';
      throw error;
    } finally {
      if (synchronous) {
        logTimeSpent();
      }
    }

    return result;
  }

  isThenable(value) {
    return typeof value === 'object' && value !== null && typeof value.then === 'function';
  }

}

class Namespace {
  constructor(metricClient, domain, module) {
    this.EmptyValue = '';
    this.metricClient = void 0;
    this.domain = void 0;
    this.module = void 0;
    this.metrics = void 0;
    this.attributes = void 0;
    this.metricClient = metricClient;
    this.domain = domain != null ? domain : this.EmptyValue;
    this.module = module != null ? module : this.EmptyValue;
    this.metrics = new Map();
    this.attributes = {};
  }

  getMetricClient() {
    return this.metricClient;
  }

  getDomain() {
    return this.domain;
  }

  getModule() {
    return this.module;
  }

  getNamespacePrefix() {
    if (this.isEmptyNamespace()) {
      return '';
    }

    return `${this.domain}_${this.module}_`;
  }

  getAttributes() {
    return this.attributes;
  }

  getAttribute(attributeName) {
    return this.attributes[attributeName];
  }

  setAttribute(attributeName, attributeValue) {
    this.attributes[attributeName] = attributeValue;
  }

  removeAttribute(attributeName) {
    delete this.attributes[attributeName];
  }

  getMetric(metricName) {
    try {
      return this.metrics.get(this.getMetricKey(metricName));
    } catch (_unused) {
      return undefined;
    }
  }

  createGauge(metricName, description, unit, attributes = {}) {
    const gauge = new Gauge(this, metricName, description, unit, attributes);
    this.metrics.set(this.getMetricKey(metricName), gauge);
    return gauge;
  }

  recordGauge(metricName, value, attributes) {
    const gauge = this.getGauge(metricName);

    if (gauge === undefined) {
      Logger.getLogger().warn(Warnings.MetricSkipped);
      return;
    }

    gauge.record(value, attributes);
  }

  createCounter(metricName, description, unit, attributes = {}) {
    const counter = new Counter(this, metricName, description, unit, attributes);
    this.metrics.set(this.getMetricKey(metricName), counter);
    return counter;
  }

  incrementCounter(metricName, value, attributes) {
    const counter = this.getCounter(metricName);

    if (counter === undefined) {
      Logger.getLogger().warn(Warnings.MetricSkipped);
      return;
    }

    counter.increment(value, attributes);
  }

  createHistogram(metricName, description, unit, buckets, attributes = {}) {
    const histogram = new Histogram(this, metricName, description, unit, buckets, attributes);
    this.metrics.set(this.getMetricKey(metricName), histogram);
    return histogram;
  }

  recordHistogram(metricName, value, buckets, attributes) {
    const histogram = this.getHistogram(metricName);

    if (histogram === undefined) {
      Logger.getLogger().warn(Warnings.MetricSkipped);
      return;
    }

    histogram.record(value, buckets, attributes);
  }

  createTimer(metricName, description, unit, buckets, attributes = {}) {
    const timer = new Timer(this, metricName, description, unit, buckets, attributes);
    this.metrics.set(this.getMetricKey(metricName), timer);
    return timer;
  }

  recordTimer(metricName, value, buckets, attributes) {
    const timer = this.getTimer(metricName);

    if (timer === undefined) {
      Logger.getLogger().warn(Warnings.MetricSkipped);
      return;
    }

    timer.record(value, buckets, attributes);
  }

  time(metricName, buckets = Constants.DefaultTimerBuckets, attributes = {}, fn, ...args1) {
    const timer = this.getTimer(metricName);

    if (timer === undefined) {
      Logger.getLogger().warn(Warnings.MetricSkipped);
      return fn(...args1);
    }

    return timer.time(buckets, attributes, fn, ...args1);
  }

  timeAndCount(metricNamePrefix, fn, buckets = Constants.DefaultTimerBuckets, attributes = {}) {
    return this.metricWrapper(metricNamePrefix, buckets, attributes, fn);
  }

  metricWrapper(metricNamePrefix, buckets = Constants.DefaultTimerBuckets, attributes = {}, fn, ...args1) {
    const startTime = new Date();
    let result;

    const logMetrics = () => {
      const combinedAttributes = _extends({}, attributes);

      combinedAttributes.success = success;
      this.incrementCounter(`${metricNamePrefix}_count`, 1, combinedAttributes);
      const endTime = new Date();
      const duration = endTime.getTime() - startTime.getTime();
      this.recordTimer(`${metricNamePrefix}_duration`, duration, buckets, combinedAttributes);
    };

    let success = 'true';
    let synchronous = true;

    try {
      result = fn(...args1);

      if (this.isThenable(result)) {
        synchronous = false;
        void result.catch(() => {
          success = 'false';
        }).then(() => logMetrics());
      }
    } catch (error) {
      success = 'false';
      throw error;
    } finally {
      if (synchronous) {
        logMetrics();
      }
    }

    return result;
  }

  isThenable(value) {
    return typeof value === 'object' && value !== null && typeof value.then === 'function';
  }

  isEmptyNamespace() {
    return this.domain === this.EmptyValue && this.module === this.EmptyValue;
  }

  getMetricKey(metricName) {
    return `${this.getNamespacePrefix()}_${metricName}`;
  }

  getGauge(metricName) {
    const metric = this.getMetric(metricName);

    if (metric === undefined) {
      return new Gauge(this, metricName);
    } else if (!(metric instanceof Gauge)) {
      Logger.getLogger().warn(Warnings.MetricTypeMismatch);
      return undefined;
    }

    return metric;
  }

  getCounter(metricName) {
    const metric = this.getMetric(metricName);

    if (metric === undefined) {
      return new Counter(this, metricName);
    } else if (!(metric instanceof Counter)) {
      Logger.getLogger().warn(Warnings.MetricTypeMismatch);
      return undefined;
    }

    return metric;
  }

  getHistogram(metricName) {
    const metric = this.getMetric(metricName);

    if (metric === undefined) {
      return new Histogram(this, metricName);
    } else if (!(metric instanceof Histogram)) {
      Logger.getLogger().warn(Warnings.MetricTypeMismatch);
      return undefined;
    }

    return metric;
  }

  getTimer(metricName) {
    const metric = this.getMetric(metricName);

    if (metric === undefined) {
      return new Timer(this, metricName);
    } else if (!(metric instanceof Timer)) {
      Logger.getLogger().warn(Warnings.MetricTypeMismatch);
      return undefined;
    }

    return metric;
  }

}

class MetricClient {
  constructor(isEnabled = true) {
    this.isEnabled = void 0;
    this.defaultNamespace = void 0;
    this.namespaces = void 0;
    this.attributes = void 0;
    this.defaultNamespace = undefined;
    this.namespaces = new Map();
    this.attributes = {};
    this.isEnabled = isEnabled;
    this.setDefaultHostnameAttribute();
  }

  getDefaultNamespace() {
    if (this.defaultNamespace === undefined) {
      Logger.getLogger().warn(Warnings.NoDefaultNamespaceSet);
      return undefined;
    }

    return this.defaultNamespace;
  }

  setDefaultNamespace(domain, module) {
    this.defaultNamespace = this.getNamespace(domain, module);
  }

  getNamespace(domain, module) {
    const namespaceKey = this.getNamespaceKey(domain, module);
    let namespace = this.namespaces.get(namespaceKey);

    if (namespace === undefined) {
      namespace = new Namespace(this, domain, module);
      this.namespaces.set(namespaceKey, namespace);
    }

    return namespace;
  }

  getAttributes() {
    return this.attributes;
  }

  getAttribute(attributeName) {
    return this.attributes[attributeName];
  }

  setAttribute(attributeName, attributeValue) {
    this.attributes[attributeName] = attributeValue;
  }

  removeAttribute(attributeName) {
    delete this.attributes[attributeName];
  }

  incrementCounter(metricName, value, attributes) {
    if (!this.isEnabled) {
      return;
    }

    this.counterImpl(metricName, value, attributes);
  }

  recordGauge(metricName, value, attributes) {
    if (!this.isEnabled) {
      return;
    }

    this.gaugeImpl(metricName, value, attributes);
  }

  recordTimer(metricName, value, buckets, attributes) {
    if (!this.isEnabled) {
      return;
    }

    this.timeImpl(metricName, value, buckets, attributes);
  }

  recordHistogram(metricName, value, buckets, attributes) {
    if (!this.isEnabled) {
      return;
    }

    this.histogramImpl(metricName, value, buckets, attributes);
  }

  getNamespaceKey(domain, module) {
    return `${domain}_${module}`;
  }

  setDefaultHostnameAttribute() {
    try {
      this.setAttribute('hostname', os.hostname());
    } catch (_unused) {
      Logger.getLogger().warn(Warnings.FailedToSetHostname);
    }
  }

}

class StatsDClient extends MetricClient {
  constructor(hostname, portNumber, isEnabled = true, mock = false) {
    super(isEnabled);
    this.statsDClient = void 0;
    this.statsDClient = new StatsD({
      host: hostname,
      port: portNumber,
      mock: mock
    });
  }

  gaugeImpl(metricName, value, attributes) {
    const tags = this.mapAttributesToTags(attributes);
    this.statsDClient.gauge(metricName, value, undefined, tags);
  }

  counterImpl(metricName, value, attributes) {
    const tags = this.mapAttributesToTags(attributes);
    this.statsDClient.increment(metricName, value, undefined, tags);
  }

  timeImpl(metricName, value, buckets, attributes) {
    this.recordHistogram(metricName, value, buckets, attributes);
  }

  histogramImpl(metricName, value, buckets, attributes) {
    const combinedAttributes = this.appendBucketsAttribute(buckets, attributes);
    const tags = this.mapAttributesToTags(combinedAttributes);
    this.statsDClient.timing(metricName, value, undefined, tags);
  }

  appendBucketsAttribute(buckets, attributes) {
    const clonedAttributes = _extends({}, attributes);

    clonedAttributes[Constants.HistogramBucketAttributeName] = buckets.join(';');
    return clonedAttributes;
  }

  mapAttributesToTags(attributes) {
    if (attributes === undefined) {
      return {};
    }

    return this.sanitizeAttributes(attributes);
  }

  sanitizeAttributes(attributes) {
    const sanitizedAttributes = {};

    for (const attributeName in attributes) {
      if (attributes[attributeName]) {
        sanitizedAttributes[attributeName] = attributes[attributeName].replace(/[#,:]/g, '_');
      }
    }

    return sanitizedAttributes;
  }

}

class MetricClientFactory {
  static createClient(config) {
    switch (config.client.toLowerCase()) {
      case 'statsd':
        {
          return new StatsDClient(config.hostname, config.port, config.enabled);
        }

      default:
        {
          return new StatsDClient(config.hostname, config.port, config.enabled);
        }
    }
  }

}

class InMemoryClient extends MetricClient {
  constructor() {
    super();
    this.metrics = void 0;
    this.metricAttributes = void 0;
    this.metrics = new Map();
    this.metricAttributes = new Map();
  }

  getMetrics() {
    return this.metrics;
  }

  getMetricAttributes() {
    return this.metricAttributes;
  }

  gaugeImpl(metricName, value, attributes) {
    this.metrics.set(metricName, value);
    this.metricAttributes.set(metricName, attributes);
  }

  counterImpl(metricName, value, attributes) {
    let currentValue = this.metrics.get(metricName);

    if (currentValue === undefined) {
      currentValue = 0;
    }

    this.metrics.set(metricName, currentValue + value);
    this.metricAttributes.set(metricName, attributes);
  }

  timeImpl(metricName, value, buckets, attributes) {
    this.recordHistogram(metricName, value, buckets, attributes);
  }

  histogramImpl(metricName, value, _buckets, attributes) {
    this.metrics.set(metricName, value);
    this.metricAttributes.set(metricName, attributes);
  }

}

const config = Configuration.initializeConfiguration(process.env.TELEMETRY_CONFIG);
Metrics.setDefaultClient(MetricClientFactory.createClient(config.metrics));
const setDefaultClient = client => Metrics.setDefaultClient(client);
const setDefaultNamespace = (domain, module) => {
  Metrics.getMetricClient().setDefaultNamespace(domain, module);
};
const getNamespace = (domain, module) => Metrics.getMetricClient().getNamespace(domain, module);
const setAttribute = (attributeName, attributeValue) => Metrics.getMetricClient().setAttribute(attributeName, attributeValue);
const removeAttribute = attributeName => Metrics.getMetricClient().removeAttribute(attributeName);
const createGauge = (metricName, description, unit, attributes = {}) => {
  const namespace = Metrics.getMetricClient().getDefaultNamespace();

  if (namespace === undefined) {
    return undefined;
  }

  return namespace.createGauge(metricName, description, unit, attributes);
};
const recordGauge = (metricName, value, attributes = {}) => {
  const namespace = Metrics.getMetricClient().getDefaultNamespace();

  if (namespace !== undefined) {
    namespace.recordGauge(metricName, value, attributes);
  }
};
const createCounter = (metricName, description, unit, attributes = {}) => {
  const namespace = Metrics.getMetricClient().getDefaultNamespace();

  if (namespace === undefined) {
    return undefined;
  }

  return namespace.createCounter(metricName, description, unit, attributes);
};
const incrementCounter = (metricName, value = 1, attributes = {}) => {
  const namespace = Metrics.getMetricClient().getDefaultNamespace();

  if (namespace !== undefined) {
    namespace.incrementCounter(metricName, value, attributes);
  }
};
const createHistogram = (metricName, description, unit, buckets = [], attributes = {}) => {
  const namespace = Metrics.getMetricClient().getDefaultNamespace();

  if (namespace === undefined) {
    return undefined;
  }

  return namespace.createHistogram(metricName, description, unit, buckets, attributes);
};
const recordHistogram = (metricName, value, buckets = [], attributes = {}) => {
  const namespace = Metrics.getMetricClient().getDefaultNamespace();

  if (namespace !== undefined) {
    namespace.recordHistogram(metricName, value, buckets, attributes);
  }
};
const createTimer = (metricName, description, unit, buckets = [], attributes = {}) => {
  const namespace = Metrics.getMetricClient().getDefaultNamespace();

  if (namespace === undefined) {
    return undefined;
  }

  return namespace.createTimer(metricName, description, unit, buckets, attributes);
};
const recordTimer = (metricName, value, buckets = [], attributes = {}) => {
  const namespace = Metrics.getMetricClient().getDefaultNamespace();

  if (namespace !== undefined) {
    namespace.recordTimer(metricName, value, buckets, attributes);
  }
};
function time(metricName, buckets = Constants.DefaultTimerBuckets, attributes = {}, fn, ...args1) {
  const namespace = Metrics.getMetricClient().getDefaultNamespace();

  if (namespace === undefined) {
    return undefined;
  }

  return namespace.time(metricName, buckets, attributes, fn, ...args1);
}
function timeAndCount(metricNamePrefix, fn, buckets = Constants.DefaultTimerBuckets, attributes = {}) {
  const namespace = Metrics.getMetricClient().getDefaultNamespace();

  if (namespace === undefined) {
    return undefined;
  }

  return namespace.timeAndCount(metricNamePrefix, fn, buckets, attributes);
}
function metricWrapper(metricNamePrefix, buckets = Constants.DefaultTimerBuckets, attributes = {}, fn, ...args1) {
  const namespace = Metrics.getMetricClient().getDefaultNamespace();

  if (namespace === undefined) {
    return undefined;
  }

  return namespace.metricWrapper(metricNamePrefix, buckets, attributes, fn, ...args1);
}
function timeDecorator(metricName, buckets = Constants.DefaultTimerBuckets, attributes = {}) {
  return function (_target, _propertyKey, descriptor) {
    const value = descriptor.value;

    descriptor.value = function (...args) {
      return time(metricName, buckets, attributes, value, ...args);
    };

    return descriptor;
  };
}
function timeAndCountDecorator(metricNamePrefix, buckets = Constants.DefaultTimerBuckets, attributes = {}) {
  return function (_target, _propertyKey, descriptor) {
    const value = descriptor.value;

    descriptor.value = function (...args) {
      return metricWrapper(metricNamePrefix, buckets, attributes, value, ...args);
    };

    return descriptor;
  };
}

export { InMemoryClient, StatsDClient, createCounter, createGauge, createHistogram, createTimer, getNamespace, incrementCounter, metricWrapper, recordGauge, recordHistogram, recordTimer, removeAttribute, setAttribute, setDefaultClient, setDefaultNamespace, time, timeAndCount, timeAndCountDecorator, timeDecorator };
//# sourceMappingURL=index.modern.js.map
